from concurrent import futures

import grpc

from grebble_flow.grpc.processor import processor_pb2_grpc
from grebble_flow.grpc.processor.service import ProcessorService


def start_server(port):
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    processor_pb2_grpc.add_ProcessorServicer_to_server(ProcessorService(), server)
    server.add_insecure_port(f"[::]:{port}")
    server.start()
    print('Server started')
    server.wait_for_termination()



import json

from grebble_flow.grpc.processor import processor_pb2_grpc, processor_pb2
from grebble_flow.managment.manager import FlowManager


class ProcessorService(processor_pb2_grpc.ProcessorServicer):
    def __init__(self, *args, **kwargs):
        self.flow_manager = FlowManager()

    def Execute(self, request, context):
        # get the string from the incoming request
        flow_name = request.flowName
        json_string = request.jsonString

        for item in self.flow_manager.run(flow_name, json.loads(json_string)):
            yield processor_pb2.FlowResponse(
                jsonString=json.dumps(item), streamEnd=False
            )

        yield processor_pb2.FlowResponse(jsonString={}, streamEnd=True)



from pathlib import Path

import pkg_resources


def install_proto():
    import grpc_tools.protoc

    cwd = Path(pkg_resources.resource_filename("grebble_flow", "")).parents[0]
    proto_include = pkg_resources.resource_filename("grebble_flow", "grpc/processor/")
    grpc_tools.protoc.main(
        [
            "grpc_tools.protoc",
            f"--proto_path={cwd}",
            "--python_out=./",
            "--grpc_python_out=./",
            f"{proto_include}processor.proto",
        ]
    )



class BaseFlowProcessor(object):
    is_flow_processor = True

    def initialize(self, *args, **kwargs):
        raise NotImplemented()

    def execute(self, *args, **kwargs):
        raise NotImplemented()



from grebble_flow.processors.base import BaseFlowProcessor


class SimpleProcessor(BaseFlowProcessor):
    name = "simple-flow-processor"

    def execute(self, *args, **kwargs):
        for i in range(0, 100):
            yield {"test": i}
